// Copyright (C) 2016 Fime
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#ifndef USE_MPI
#define BOOST_TEST_MODULE testPortfolioSDDP
#endif
#define BOOST_TEST_DYN_LINK
#ifdef USE_MPI
#include <boost/mpi.hpp>
#endif
#include <functional>
#include <array>
#include <memory>
#define _USE_MATH_DEFINES
#include <math.h>
#include <boost/test/unit_test.hpp>
#include <Eigen/Dense>

using namespace std;
using namespace Eigen ;
using namespace StOpt;

/// \file testPortfolioSDDP.cpp
/// Define a test case with N assets for portfolio optimization with SDDP.
/// A portfolio is invested in N risky assets \f$S_i\f$ and a none risky asset.
/// The none risky asset has a zero yield.
/// The risky assets \f$ S^i \f$ has a stochastic yield into two dates \f$ t- dt\d$ and \f$ t \$ :
///    \f$ a^i_t = \mu^i + \sigma^i g^i_t \f$ where the \f$ g^i_t \f$ at a given date \f$ t\f$
/// are centered unit gaussian with correlations matrice \$f \Sigma \f$
/// so that $S^i_t = (1+a^i_t) S^i_{t-dt}$
/// \f$ X_t= (X^1_t,..,X^N,t, ...., X^{3N}_t, X^{3N+1}_t \f$ is the state vector.
///  - \f$ X^i_t,  i \[1,N] \f$ represents the positive amount invested in asset \f$i\f$ at date \f$t\f$ before rebalancing
///  - \f$ X^i_t, i \in [N+1,2N]\f$ represents the positive amount of assets \f$i-N\f$ bought  at date \f$t\f$
///  - \f$ X^i_t, i \in [2N+1,3N]\f$ represents the positive amount of assets \f$i-2N\f$ sold  at date \f$t\f$
///  - \f$ X^{3N+1}_t\f$ is the positive amount invested in the non risky asset at date \f$t\f$ before rebalancing
/// \f$ c \f$ rpresents the transaction cost paid at sell or buy orders.
/// The flow equations are
/// - \f$ X^i_t = a^i_t (X^i_{t-dt} - X^{i+N}_{t-dt} + X^{i+2N}_{t-dt}) \f$
/// - \f$ X^{3N+1}_t =  X^{3N+1}_{t-dt} + (1-c)  X^{i+N}_{t-dt}  -(1+c) X^{i+2N}_{t-dt} \f$
/// The cost function to minimize
/// \f$ E[ \sum_{t>0} ( -  \sum_i (a^i_t X^i_{t-dt} + c X^{i+N}_{t-dt} + c X^{i+2N}_{t_dt}) ) ] \f$

