// Copyright (C) 2016 EDF
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#include <Eigen/Dense>
#include <iostream>
#include <memory>
#include <boost/version.hpp>
#include <boost/bind.hpp>
#include "test/c++/tools/simulators/SimulatorGaussianSDDP.h"

/** \file BoostPythonSDDPSimulators.cpp
 * \brief Map Simulators for SDDP
 * \author Xavier Warin
 */


using namespace Eigen;
using namespace std;
using namespace StOpt;

#ifdef _DEBUG
#undef _DEBUG
#include <Python.h>
#define _DEBUG
#else
#include <Python.h>
#endif
#include <numpy/arrayobject.h>
#include "StOpt/python/NumpyConverter.hpp"

using namespace boost::python;



struct SimulatorGaussianSDDPWrap: SimulatorGaussianSDDP, wrapper<SimulatorGaussianSDDP>
{

    inline  void updateDates(const double &p_date)
    {
        if (override py_override = this->get_override("updateDates"))
        {
            this->get_override("updateDates")(p_date);
        }
        SimulatorGaussianSDDP::updateDates(p_date);
    }
    inline void defaultUpdateDates(const double &p_date)
    {
        SimulatorGaussianSDDP::updateDates(p_date);
    }
};

// MSVC 2015 BUG
#if (_MSC_VER == 1900)
namespace boost
{

template <>
SimulatorGaussianSDDP  const volatile *get_pointer< class  SimulatorGaussianSDDP const volatile >(
        class SimulatorGaussianSDDP  const volatile *c)
{
    return c;
}

template <>
SimulatorGaussianSDDPWrap  const volatile *get_pointer< class  SimulatorGaussianSDDPWrap const volatile >(
        class SimulatorGaussianSDDPWrap  const volatile *c)
{
    return c;
}

}
#endif


/// \brief Encapsulation for simulators
BOOST_PYTHON_MODULE(SDDPSimulators)
{
    Register<MatrixXd>();
    Register<VectorXd>();

    class_<SimulatorGaussianSDDP, bases<SimulatorSDDPBase>  >("SimulatorGaussianSDDP", init< const int &, const int &>())
    .def(init< const int &>())
    .def("updateDates", &SimulatorGaussianSDDP::updateDates)
    ;

    class_<SimulatorGaussianSDDPWrap, std::shared_ptr<SimulatorGaussianSDDPWrap>, bases<SimulatorSDDPBase>,  boost::noncopyable  >("SimulatorGaussianSDDPWrap")
    .def("updateDates", &SimulatorGaussianSDDP::updateDates, &SimulatorGaussianSDDPWrap::defaultUpdateDates)
    ;
}
